"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));
var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));
var _lodashEs = require("lodash-es");
var _const = require("../const");
var _lexer = _interopRequireDefault(require("./lexer"));
var lexer = new _lexer.default();
var Parser = /*#__PURE__*/function () {
  function Parser() {
    (0, _classCallCheck2.default)(this, Parser);
    this.yy = {};
  }
  (0, _createClass2.default)(Parser, [{
    key: "performAction",
    value: function performAction(yytext, _yyleng, _yylineno, _yy, yystate, $$) {
      var $0 = $$.length - 1;
      switch (yystate) {
        case 1:
          // replace escaped characters with actual character
          this.$ = yytext.replace(/\\(\\|")/g, '$1').replace(/\\n/g, '\n').replace(/\\r/g, '\r').replace(/\\t/g, '\t').replace(/\\v/g, '\v').replace(/\\f/g, '\f').replace(/\\b/g, '\b');
          break;
        case 2:
          this.$ = Number(yytext);
          break;
        case 3:
          this.$ = null;
          break;
        case 4:
          this.$ = true;
          break;
        case 5:
          this.$ = false;
          break;
        case 6:
          this.$ = $$[$0 - 1];
          return this.$;
        case 13:
          this.$ = {};
          break;
        case 14:
          this.$ = $$[$0 - 1];
          break;
        case 15:
          this.$ = [$$[$0 - 2], $$[$0]];
          break;
        case 16:
          this.$ = {};
          this.$[$$[$0][0]] = $$[$0][1];
          break;
        case 17:
          this.$ = $$[$0 - 2];
          $$[$0 - 2][$$[$0][0]] = $$[$0][1];
          break;
        case 18:
          this.$ = [];
          break;
        case 19:
          this.$ = $$[$0 - 1];
          break;
        case 20:
          this.$ = [$$[$0]];
          break;
        case 21:
          this.$ = $$[$0 - 2];
          $$[$0 - 2].push($$[$0]);
          break;
        default:
          break;
      }
    }

    // eslint-disable-next-line @typescript-eslint/no-unused-vars
  }, {
    key: "parseError",
    value: function parseError(str, _hash) {
      throw new Error(str);
    }
  }, {
    key: "parse",
    value: function parse(input) {
      var _lStack, _lStack2, _lStack3, _lStack4;
      var stack = [0];
      var vStack = [null]; // semantic value stack
      var lStack = []; // location stack
      var yyText = '';
      var yyLineNo = 0;
      var yyLength = 0;
      var recovering = 0;
      lexer.setInput(input);
      lexer.yy = this.yy;
      this.yy.lexer = lexer;
      if ((0, _lodashEs.isUndefined)(lexer.yylloc)) {
        lexer.yylloc = {};
      }
      var yyloc = lexer.yylloc;
      lStack.push(yyloc);
      if ((0, _lodashEs.isFunction)(this.yy.parseError)) {
        this.parseError = this.yy.parseError;
      }
      function popStack(n) {
        stack.length -= 2 * n;
        vStack.length -= n;
        lStack.length -= n;
      }
      function lex() {
        var token;
        token = lexer.lex() || _const.EOF; // $end = 1

        // if token isn't its numeric value, convert
        if (!(0, _lodashEs.isFinite)(token)) {
          token = _const.PARSER_SYMBOLS[token] || token;
        }
        return token;
      }
      var yyVal = {};
      var symbol;
      var preErrorSymbol;
      var state;
      var action;
      var r;
      var p;
      var len;
      var newState;
      var expected;
      var errStr;
      while (true) {
        // eslint-disable-line
        // retrieve state number from top of stack
        state = stack[stack.length - 1];

        // use default actions if available
        if (_const.PARSER_DEFAULT_ACTIONS[state]) {
          action = _const.PARSER_DEFAULT_ACTIONS[state];
        } else {
          var _PARSER_TABLE$state;
          if (symbol === null || symbol === undefined) {
            symbol = lex();
          }

          // eslint-disable-next-line @typescript-eslint/ban-ts-comment
          // @ts-ignore
          action = (_PARSER_TABLE$state = _const.PARSER_TABLE[state]) === null || _PARSER_TABLE$state === void 0 ? void 0 : _PARSER_TABLE$state[symbol]; // read action for current state and first input
        }

        // handle parse error
        if ((0, _lodashEs.isUndefined)(action) || !action.length || !action[0]) {
          var _PARSER_TABLE$state2;
          if (!recovering) {
            // Report error
            expected = [];
            for (p in _const.PARSER_TABLE[state]) {
              // eslint-disable-next-line @typescript-eslint/ban-ts-comment
              // @ts-ignore
              if (_const.PARSER_TERMINALS[p] && p > 2) {
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                expected.push("'".concat(_const.PARSER_TERMINALS[p], "'"));
              }
            }
            if (lexer.showPosition) {
              // eslint-disable-next-line @typescript-eslint/ban-ts-comment
              // @ts-ignore
              errStr = "Parse error on line ".concat(yyLineNo + 1, ":\n").concat(lexer.showPosition(), "\nExpecting ").concat(expected.join(', '), ", got '").concat(_const.PARSER_TERMINALS[symbol], "'");
            } else {
              // eslint-disable-next-line @typescript-eslint/ban-ts-comment
              // @ts-ignore
              errStr = "Parse error on line ".concat(yyLineNo + 1, ": Unexpected ").concat(symbol === 1 /* EOF */ ? 'end of input' : "'".concat(_const.PARSER_TERMINALS[symbol] || symbol, "'"));
            }
            this.parseError(errStr, {
              text: lexer.match,
              token: _const.PARSER_TERMINALS[symbol] || symbol,
              line: lexer.yylineno,
              loc: yyloc,
              expected: expected
            });
          }

          // just recovered from another error
          if (recovering === 3) {
            if (symbol === _const.EOF) {
              throw new Error(errStr || 'Parsing halted.');
            }

            // discard current lookahead and grab another
            yyLength = lexer.yyleng;
            yyText = lexer.yytext;
            yyLineNo = lexer.yylineno;
            yyloc = lexer.yylloc;
            symbol = lex();
          }

          // try to recover from error
          while (true) {
            // eslint-disable-line no-constant-condition
            // check for error recovery rule in this state
            if (_const.ERROR.toString() in _const.PARSER_TABLE[state]) {
              break;
            }
            if (state === 0) {
              throw new Error(errStr || 'Parsing halted.');
            }
            popStack(1);
            state = stack[stack.length - 1];
          }
          preErrorSymbol = symbol; // save the lookahead token
          symbol = _const.ERROR; // insert generic error symbol as new lookahead
          // eslint-disable-next-line @typescript-eslint/ban-ts-comment
          // @ts-ignore
          state = stack[stack.length - 1];
          action = (_PARSER_TABLE$state2 = _const.PARSER_TABLE[state]) === null || _PARSER_TABLE$state2 === void 0 ? void 0 : _PARSER_TABLE$state2[_const.ERROR];
          recovering = 3; // allow 3 real symbols to be shifted before reporting a new error
        }

        // this shouldn't happen, unless resolve defaults are off
        if ((0, _lodashEs.isArray)(action[0]) && action.length > 1) {
          throw new Error("Parse Error: multiple actions possible at state: ".concat(state, ", token: ").concat(symbol));
        }
        switch (action[0]) {
          case 1:
            // shift
            stack.push(symbol);
            vStack.push(lexer.yytext);
            lStack.push(lexer.yylloc);
            stack.push(action[1]); // push state
            symbol = null;
            if (!preErrorSymbol) {
              // normal execution/no error
              yyLength = lexer.yyleng;
              yyText = lexer.yytext;
              yyLineNo = lexer.yylineno;
              yyloc = lexer.yylloc;
              if (recovering > 0) {
                recovering -= 1;
              }
            } else {
              // error just occurred, resume old lookahead f/ before error
              symbol = preErrorSymbol;
              preErrorSymbol = null;
            }
            break;
          case 2:
            // reduce
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            len = _const.PARSER_PRODUCTIONS[action[1]][1];

            // perform semantic action
            yyVal.$ = vStack[vStack.length - len]; // default to $$ = $1
            // default location, uses first token for firsts, last for lasts
            yyVal._$ = {
              firstLine: (_lStack = lStack[lStack.length - (len || 1)]) === null || _lStack === void 0 ? void 0 : _lStack.firstLine,
              lastLine: (_lStack2 = lStack[lStack.length - 1]) === null || _lStack2 === void 0 ? void 0 : _lStack2.lastLine,
              firstColumn: (_lStack3 = lStack[lStack.length - (len || 1)]) === null || _lStack3 === void 0 ? void 0 : _lStack3.firstColumn,
              lastColumn: (_lStack4 = lStack[lStack.length - 1]) === null || _lStack4 === void 0 ? void 0 : _lStack4.lastColumn
            };
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            r = this.performAction.call(yyVal, yyText, yyLength, yyLineNo, this.yy, action[1], vStack, lStack);
            if (!(0, _lodashEs.isUndefined)(r)) {
              return r;
            }

            // pop off stack
            if (len) {
              stack = stack.slice(0, -1 * len * 2);
              vStack = vStack.slice(0, -1 * len);
              lStack = lStack.slice(0, -1 * len);
            }

            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            stack.push(_const.PARSER_PRODUCTIONS[action[1]][0]); // push non-terminal (reduce)
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            vStack.push(yyVal.$);
            lStack.push(yyVal._$);
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            newState = _const.PARSER_TABLE[stack[stack.length - 2]][stack[stack.length - 1]];
            stack.push(newState);
            break;
          case 3:
            // accept
            return true;
          default:
            break;
        }
      }
    }
  }]);
  return Parser;
}();
var _default = new Parser();
exports.default = _default;