import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import { isUndefined as _isUndefined } from 'lodash-es';
import { EOF, LEXER_RULES, LEXER_CONDITIONS } from '../const';
import performAction from './perform-action';
var Lexer = /*#__PURE__*/function () {
  function Lexer() {
    _classCallCheck(this, Lexer);
    this.options = {};
    this._input = '';
    this.done = false;
    this.match = '';
    this.matched = '';
    this.conditionStack = ['INITIAL'];
    this.yylineno = 0;
    this.yyleng = 0;
    this.yytext = '';
    this.yylloc = {
      firstLine: 1,
      firstColumn: 0,
      lastLine: 1,
      lastColumn: 0
    };
  }
  _createClass(Lexer, [{
    key: "parseError",
    value:
    // constructor() {}

    function parseError(message, hash) {
      var _this$yy;
      if ((_this$yy = this.yy) !== null && _this$yy !== void 0 && _this$yy.parseError) {
        this.yy.parseError(message, hash);
      } else {
        throw new Error(message);
      }
    }
  }, {
    key: "setInput",
    value: function setInput(input) {
      this._input = input;
      this.done = false;
      this.matched = '';
      this.match = '';
      this.conditionStack = ['INITIAL'];
      this.yylineno = 0;
      this.yyleng = 0;
      this.yytext = '';
      this.yylloc = {
        firstLine: 1,
        firstColumn: 0,
        lastLine: 1,
        lastColumn: 0
      };
    }
  }, {
    key: "input",
    value: function input() {
      var ch = this._input[0] || '';
      this.yytext += ch;
      this.yyleng += 1;
      this.match += ch;
      this.matched += ch;
      var lines = ch.match(/\n/);
      if (lines) {
        this.yylineno += 1;
      }
      this._input = this._input.slice(1);
      return ch;
    }
  }, {
    key: "less",
    value: function less(n) {
      this._input = this.match.slice(n) + this._input;
    }
  }, {
    key: "pastInput",
    value: function pastInput() {
      var past = this.matched.substr(0, this.matched.length - this.match.length);
      return (past.length > 20 ? '...' : '') + past.substr(-20).replace(/\n/g, '');
    }
  }, {
    key: "upcomingInput",
    value: function upcomingInput() {
      var next = this.match;
      if (next.length < 20) {
        next += this._input.substr(0, 20 - next.length);
      }
      return (next.substr(0, 20) + (next.length > 20 ? '...' : '')).replace(/\n/g, '');
    }
  }, {
    key: "showPosition",
    value: function showPosition() {
      var pre = this.pastInput();
      var c = new Array(pre.length + 1).join('-');
      return "".concat(pre + this.upcomingInput(), "\n").concat(c, "^");
    }
  }, {
    key: "next",
    value: function next() {
      if (this.done) {
        return EOF;
      }
      if (!this._input) {
        this.done = true;
      }
      var token;
      var match;
      var tempMatch;
      var index;
      var lines;
      this.yytext = '';
      this.match = '';
      var rules = this._currentRules();
      for (var i = 0; i < rules.length; i++) {
        tempMatch = this._input.match(LEXER_RULES[rules[i]]);
        if (tempMatch && (!match || tempMatch[0].length > match[0].length)) {
          match = tempMatch;
          index = i;
          if (!this.options.flex) {
            break;
          }
        }
      }
      if (match) {
        lines = match[0].match(/\n.*/g);
        if (lines) {
          this.yylineno += lines.length;
        }
        this.yylloc = {
          firstLine: this.yylloc.lastLine,
          lastLine: this.yylineno + 1,
          firstColumn: this.yylloc.lastColumn,
          lastColumn: lines ? lines[lines.length - 1].length - 1 : this.yylloc.lastColumn + match[0].length
        };
        this.yytext += match[0];
        this.match += match[0];
        this.yyleng = this.yytext.length;
        this._input = this._input.slice(match[0].length);
        this.matched += match[0];
        token = performAction(this.yy, this, rules[index]); // , this.conditionStack[this.conditionStack.length - 1]

        if (this.done && this._input) {
          this.done = false;
        }
        if (token) {
          return token;
        }
        return;
      }
      if (this._input === '') {
        return EOF;
      }
      this.parseError("Lexical error on line ".concat(this.yylineno + 1, ". Unrecognized text.\n").concat(this.showPosition()), {
        text: '',
        token: null,
        line: this.yylineno
      });
    }
  }, {
    key: "lex",
    value: function lex() {
      var r = this.next();
      return _isUndefined(r) ? this.lex() : r;
    }
  }, {
    key: "_currentRules",
    value: function _currentRules() {
      var _LEXER_CONDITIONS;
      return ((_LEXER_CONDITIONS = LEXER_CONDITIONS[this.conditionStack[this.conditionStack.length - 1]]) === null || _LEXER_CONDITIONS === void 0 ? void 0 : _LEXER_CONDITIONS.rules) || [];
    }
  }]);
  return Lexer;
}();
export { Lexer as default };